@@ -36,6 +36,8 @@ class SubfolderBuildConfig:
     - Always ensures [build-system] section uses hatchling (replaces any existing build-system
       configuration from parent or subfolder)
     - Handles README files similarly (uses subfolder README if present)
+    - **Never modifies the root pyproject.toml**: The original file is temporarily moved to
+      pyproject.toml.original and restored after the build, ensuring the original is never modified
     """
 
     def __init__(
@@ -66,6 +68,7 @@ class SubfolderBuildConfig:
         self.dependency_group = dependency_group
         self.temp_pyproject: Path | None = None
         self.original_pyproject_backup: Path | None = None
+        self.original_pyproject_path: Path | None = None
         self._temp_init_created = False
         self.temp_readme: Path | None = None
         self.original_readme_backup: Path | None = None
@@ -205,6 +208,10 @@ class SubfolderBuildConfig:
         The [build-system] section is always set to use hatchling, even if the parent or
         subfolder pyproject.toml uses a different build backend (e.g., setuptools).
 
+        **Important**: The root pyproject.toml is never modified. Instead, it is temporarily
+        moved to pyproject.toml.original and restored after the build completes. This ensures
+        the original file remains unchanged.
+
         Returns:
             Path to the pyproject.toml file (either from subfolder or created temporary),
             or None if no parent pyproject.toml exists (in which case subfolder config is skipped)
@@ -228,20 +235,33 @@ class SubfolderBuildConfig:
             print(f"Using existing pyproject.toml from subfolder: {subfolder_pyproject}")
             self._used_subfolder_pyproject = True
 
-            # Backup the original project root pyproject.toml if it exists
+            # Store reference to original project root pyproject.toml
             original_pyproject = self.project_root / "pyproject.toml"
-            if original_pyproject.exists():
-                backup_path = self.project_root / "pyproject.toml.backup"
-                shutil.copy2(original_pyproject, backup_path)
-                self.original_pyproject_backup = backup_path
+            self.original_pyproject_path = original_pyproject
+
+            # Create temporary pyproject.toml file
+            temp_pyproject_path = self.project_root / "pyproject.toml.temp"
 
             # Read and adjust the subfolder pyproject.toml
             subfolder_content = subfolder_pyproject.read_text(encoding="utf-8")
             # Adjust packages path to be relative to project root
             adjusted_content = self._adjust_subfolder_pyproject_packages_path(subfolder_content)
 
-            # Write adjusted content to project root
-            original_pyproject.write_text(adjusted_content, encoding="utf-8")
+            # Write adjusted content to temporary file
+            temp_pyproject_path.write_text(adjusted_content, encoding="utf-8")
+            self.temp_pyproject = temp_pyproject_path
+
+            # If original pyproject.toml exists, temporarily move it
+            if original_pyproject.exists():
+                backup_path = self.project_root / "pyproject.toml.original"
+                # Remove backup if it already exists (from previous failed test or run)
+                if backup_path.exists():
+                    backup_path.unlink()
+                original_pyproject.rename(backup_path)
+                self.original_pyproject_backup = backup_path
+
+            # Move temp file to pyproject.toml for the build
+            temp_pyproject_path.rename(original_pyproject)
             self.temp_pyproject = original_pyproject
 
             # Handle README file
@@ -270,9 +290,15 @@ class SubfolderBuildConfig:
 
         original_content = original_pyproject.read_text(encoding="utf-8")
 
-        # Create a backup
-        backup_path = self.project_root / "pyproject.toml.backup"
-        shutil.copy2(original_pyproject, backup_path)
+        # Store reference to original
+        self.original_pyproject_path = original_pyproject
+
+        # Temporarily move original to backup location
+        backup_path = self.project_root / "pyproject.toml.original"
+        # Remove backup if it already exists (from previous failed test or run)
+        if backup_path.exists():
+            backup_path.unlink()
+        original_pyproject.rename(backup_path)
         self.original_pyproject_backup = backup_path
 
         # Parse and modify the pyproject.toml
@@ -327,8 +353,12 @@ class SubfolderBuildConfig:
                 original_content, parent_dependency_group
             )
 
-        # Write the modified content
-        original_pyproject.write_text(modified_content, encoding="utf-8")
+        # Write the modified content to a temporary file
+        temp_pyproject_path = self.project_root / "pyproject.toml.temp"
+        temp_pyproject_path.write_text(modified_content, encoding="utf-8")
+
+        # Move temp file to pyproject.toml for the build
+        temp_pyproject_path.rename(original_pyproject)
         self.temp_pyproject = original_pyproject
 
         # Handle README file
@@ -535,6 +565,139 @@ class SubfolderBuildConfig:
 
         return "\n".join(result)
 
+    def add_third_party_dependencies(self, dependencies: list[str]) -> None:
+        """
+        Add third-party dependencies to the temporary pyproject.toml.
+
+        This method updates the pyproject.toml file that was created for the subfolder
+        build by adding the specified dependencies to the [project.dependencies] section.
+
+        Args:
+            dependencies: List of third-party package names to add (e.g., ["pypdf", "requests"])
+        """
+        if not self.temp_pyproject or not self.temp_pyproject.exists():
+            print(
+                f"Warning: Cannot add third-party dependencies - pyproject.toml not found at {self.temp_pyproject}",
+                file=sys.stderr,
+            )
+            return
+
+        if not dependencies:
+            return
+
+        print(f"Adding third-party dependencies to pyproject.toml: {', '.join(dependencies)}")
+        content = self.temp_pyproject.read_text(encoding="utf-8")
+        updated_content = self._add_dependencies_to_pyproject(content, dependencies)
+        self.temp_pyproject.write_text(updated_content, encoding="utf-8")
+
+    def _normalize_package_name(self, package_name: str) -> str:
+        """
+        Normalize package name for PyPI.
+
+        Converts underscores to hyphens, as PyPI package names typically use hyphens
+        while Python import names use underscores (e.g., 'better_enum' -> 'better-enum').
+
+        Args:
+            package_name: Package name from import statement
+
+        Returns:
+            Normalized package name for PyPI
+        """
+        # Convert underscores to hyphens for PyPI package names
+        # This handles the common case where import names use underscores
+        # but PyPI package names use hyphens
+        return package_name.replace("_", "-")
+
+    def _add_dependencies_to_pyproject(self, content: str, dependencies: list[str]) -> str:
+        """
+        Add dependencies to pyproject.toml content.
+
+        Adds the specified dependencies to the [project] section's dependencies list.
+        If dependencies already exist, merges them. If no dependencies section exists,
+        creates one. Package names are normalized (underscores -> hyphens) to match
+        PyPI naming conventions.
+
+        Args:
+            content: Current pyproject.toml content
+            dependencies: List of dependency names to add (will be normalized)
+
+        Returns:
+            Updated pyproject.toml content with dependencies added
+        """
+        if not dependencies:
+            return content
+
+        # Normalize package names (convert underscores to hyphens for PyPI)
+        normalized_deps = [self._normalize_package_name(dep) for dep in dependencies]
+
+        lines = content.split("\n")
+        result = []
+        in_project = False
+        in_dependencies = False
+        dependencies_added = False
+        existing_deps: set[str] = set()
+
+        # First pass: find existing dependencies
+        for line in lines:
+            if line.strip().startswith("[project]"):
+                in_project = True
+            elif line.strip().startswith("[") and in_project:
+                in_project = False
+            elif in_project and re.match(r"^\s*dependencies\s*=\s*\[", line):
+                in_dependencies = True
+            elif in_dependencies:
+                # Extract existing dependency names
+                dep_match = re.search(r'["\']([^"\']+)["\']', line)
+                if dep_match:
+                    existing_deps.add(dep_match.group(1))
+                if line.strip().endswith("]"):
+                    in_dependencies = False
+
+        # Merge with new dependencies (normalized)
+        all_deps = sorted(existing_deps | set(normalized_deps))
+
+        # Second pass: build result with dependencies
+        in_project = False
+        in_dependencies = False
+        for line in lines:
+            if line.strip().startswith("[project]"):
+                in_project = True
+                result.append(line)
+            elif line.strip().startswith("[") and in_project:
+                # End of [project] section, add dependencies if not already present
+                if not dependencies_added:
+                    result.append("dependencies = [")
+                    for dep in all_deps:
+                        result.append(f'    "{dep}",')
+                    result.append("]")
+                    result.append("")
+                in_project = False
+                result.append(line)
+            elif in_project and re.match(r"^\s*dependencies\s*=\s*\[", line):
+                # Replace existing dependencies section
+                result.append("dependencies = [")
+                for dep in all_deps:
+                    result.append(f'    "{dep}",')
+                result.append("]")
+                dependencies_added = True
+                in_dependencies = True
+            elif in_dependencies:
+                # Skip lines in existing dependencies section (already replaced)
+                if line.strip().endswith("]"):
+                    in_dependencies = False
+            else:
+                result.append(line)
+
+        # If [project] section exists but no dependencies were added, add them
+        if in_project and not dependencies_added:
+            result.append("dependencies = [")
+            for dep in all_deps:
+                result.append(f'    "{dep}",')
+            result.append("]")
+            result.append("")
+
+        return "\n".join(result)
+
     def _handle_readme(self) -> None:
         """
         Handle README file for subfolder builds.
@@ -585,9 +748,10 @@ class SubfolderBuildConfig:
         """
         Restore the original pyproject.toml and remove temporary __init__.py if created.
 
-        If a subfolder pyproject.toml was used, it restores the original project root
-        pyproject.toml from backup. If a temporary pyproject.toml was created, it
-        restores the original as well.
+        The root pyproject.toml is never modified during subfolder builds. Instead, it is
+        temporarily moved to pyproject.toml.original and then restored after the build.
+        This method removes the temporary pyproject.toml and restores the original from
+        the backup location, ensuring the original file is never modified.
         """
         # Remove temporary __init__.py if we created it
         if self._temp_init_created:
@@ -630,16 +794,26 @@ class SubfolderBuildConfig:
             self.temp_readme = None
 
         # Restore original pyproject.toml (only if we created/used one)
-        if (
-            self.temp_pyproject
-            and self.original_pyproject_backup
-            and self.original_pyproject_backup.exists()
-        ):
-            original_pyproject = self.project_root / "pyproject.toml"
-            shutil.copy2(self.original_pyproject_backup, original_pyproject)
-            self.original_pyproject_backup.unlink()
-            self.original_pyproject_backup = None
+        if self.temp_pyproject and self.original_pyproject_path:
+            original_pyproject = self.original_pyproject_path
+
+            # Remove the temporary pyproject.toml we created
+            if original_pyproject.exists():
+                try:
+                    original_pyproject.unlink()
+                except Exception as e:
+                    print(
+                        f"Warning: Could not remove temporary pyproject.toml: {e}",
+                        file=sys.stderr,
+                    )
+
+            # Restore the original pyproject.toml from backup if it existed
+            if self.original_pyproject_backup and self.original_pyproject_backup.exists():
+                self.original_pyproject_backup.rename(original_pyproject)
+                self.original_pyproject_backup = None
+
             self.temp_pyproject = None
+            self.original_pyproject_path = None
             self._used_subfolder_pyproject = False
 
     def __enter__(self) -> Self:
